@@ -17,7 +17,7 @@
 from __future__ import absolute_import
 
 import collections
-import copy
+from copy import deepcopy
 import functools
 import re
 import types
@@ -26,6 +26,7 @@ from normalize.coll import ListCollection
 from normalize.exc import FieldSelectorAttributeError
 from normalize.exc import FieldSelectorException
 from normalize.exc import FieldSelectorKeyError
+from normalize.exc import FieldSelectorOperationUnsupported
 
 
 @functools.total_ordering
@@ -238,6 +239,47 @@ class FieldSelector(object):
         type(self)([self.selectors[-1]]).put(record, value)
         return 1
 
+    def delete(self, record):
+        """
+        Like 'put', but deletes the item from the specified location.
+
+        If there is a problem, this method will typically raise
+        :py:class:`FieldSelectorException`.
+
+        ::
+
+           record.foo = "bar"
+           field_selector = FieldSelector(["foo"])
+           field_selector.delete(record)
+           print field_selector.foo  # AttributeError
+        """
+        i = 0
+        for selector in self.selectors[:-1]:
+            if selector is None:
+                sub_field_selector = type(self)(self.selectors[i + 1:])
+                for r in record:
+                    sub_field_selector.delete(r)
+            elif isinstance(selector, (int, long)):
+                try:
+                    record = record[selector]
+                except IndexError:
+                    raise FieldSelectorKeyError(key=selector)
+            else:
+                if not hasattr(record, selector):
+                    raise FieldSelectorAttributeError(name=selector)
+                record = getattr(record, selector)
+            i = i + 1
+        to_delete = self.selectors[-1]
+        if isinstance(to_delete, (int, long, types.NoneType)):
+            # can't do this until the coll types support mutation operations
+            raise FieldSelectorOperationUnsupported(
+                operation="delete",
+                target="collection",
+            )
+        else:
+            if hasattr(record, to_delete):
+                delattr(record, to_delete)
+
     def __eq__(self, other):
         """Implemented; field selectors must have identical paths to compare
         equal."""
@@ -371,6 +413,8 @@ class FieldSelector(object):
 
     @classmethod
     def from_path(cls, path):
+        """Alternate constructor.  Constructs a FieldSelector from the
+        abbreviated text representation (.path attribute)"""
         return cls(_scan_selector_path(path))
 
 
@@ -426,6 +470,10 @@ def _fmt_mfs_path(head, tail):
     )
 
 
+class _None(object):
+    pass
+
+
 class MultiFieldSelector(object):
     """Version of a FieldSelector which stores multiple FieldSelectors combined
     into a single tree structure."""
@@ -610,7 +658,7 @@ class MultiFieldSelector(object):
 
     def _get(self, obj, tail):
         if tail is all:
-            return copy.deepcopy(obj)
+            return deepcopy(obj)
         else:
             return tail.get(obj)
 
@@ -665,3 +713,126 @@ class MultiFieldSelector(object):
                     if val is not None:
                         kwargs[head] = self._get(val, tail)
                 return ctor(**kwargs)
+
+    def delete(self, obj, force=False):
+        """Deletes all of the fields at the specified locations.
+
+        args:
+
+            ``obj=``\ *OBJECT*
+                the object to remove the fields from
+
+            ``force=``\ *BOOL*
+                if True, missing attributes do not raise errors.  Otherwise,
+                the first failure raises an exception without making any
+                changes to ``obj``.
+        """
+        # TODO: this could be a whole lot more efficient!
+        if not force:
+            for fs in self:
+                try:
+                    fs.get(obj)
+                except FieldSelectorException:
+                    raise
+
+        for fs in self:
+            try:
+                fs.delete(obj)
+            except FieldSelectorException:
+                pass
+
+    def patch(self, target, source, copy=False):
+        """Copies fields from ``obj`` to ``target``.  If a matched field does
+        not exist in ``obj``, it will be deleted from ``target``, otherwise it
+        will be assigned (or copied).
+
+        args:
+
+            ``target=``\ *OBJECT*
+                the object to set the fields in
+
+            ``source=``\ *OBJECT*
+                the object to lift the fields from
+
+            ``copy=``\ *BOOL*\ |\ *FUNCTION*
+                deep copy the values set, using copy.deepcopy (or the passed
+                function).  False by default.
+        """
+        # TODO: this could also be a whole lot more efficient!
+        fs_val = []
+        for fs in self:
+            try:
+                fs_val.append((fs, fs.get(source)))
+            except AttributeError:
+                fs_val.append((fs, _None))
+            except FieldSelectorException:
+                raise
+
+        if copy and not callable(copy):
+            copy = lambda x: deepcopy(x)
+
+        for fs, val in fs_val:
+            if val is _None:
+                fs.delete(target)
+            else:
+                fs.post(target, val if not copy else copy(val))
+
+    @classmethod
+    def from_path(cls, mfs_path):
+        """Alternate constructor.  Constructs a MultiFieldSelector from the
+        abbreviated text representation (.path attribute)"""
+        mfs = _scan_mfs_path(mfs_path)
+        return cls(*mfs)
+
+
+_MFS_PATH_TOK = re.compile(
+    r'''\.(?P<attr>\w+)|
+        \[(?:
+            (?P<idx>\d+)|
+            (?P<wild>\*)|
+            '(?P<key>(?:[^']+|\\')*)'
+        )\]|
+        (?P<branch>[()|])
+        ''',
+    re.X,
+)
+
+
+def _scan_mfs_path(path):
+    mfs = list()
+    fs = list()
+    stack = list()
+    popping = False
+    for m in re.finditer(_MFS_PATH_TOK, path):
+        if m.group("branch"):
+            branch = m.group("branch")
+            if branch == "(":
+                stack.append(list(fs))
+            elif branch == "|":
+                if not popping:
+                    mfs.append(fs)
+                fs = list(stack[-1])
+            else:
+                if not popping:
+                    mfs.append(fs)
+                    popping = True
+                fs = stack.pop()
+        else:
+            popping = False
+
+        text_item = m.group('key')
+        if text_item is not None:
+            text_item = re.sub(r'\\(.)', lambda m: m.group(1), text_item)
+        else:
+            text_item = m.group('attr')
+
+        if text_item:
+            fs.append(text_item)
+        else:
+            idx_item = m.group('idx')
+            if idx_item:
+                fs.append(int(idx_item))
+            elif m.group('wild'):
+                fs.append(None)
+
+    return mfs
